@@ -9,11 +9,11 @@
     };
     int packobjc[] = {
         102,
-        189,
-        101,
-        87,
-        499,
-        206
+        191,
+        106,
+        88,
+        500,
+        209
     };
     const char *packobj[] = {
         "AirPassengers",
@@ -144,8 +144,10 @@
         "URLdecode",
         "URLencode",
         "View",
+        "adist",
         "alarm",
         "apropos",
+        "aregexec",
         "argsAnywhere",
         "as.person",
         "as.personList",
@@ -317,6 +319,7 @@
         "adjustcolor",
         "as.graphicsAnnot",
         "as.raster",
+        "axisTicks",
         "bitmap",
         "blues9",
         "bmp",
@@ -337,11 +340,15 @@
         "contourLines",
         "convertColor",
         "densCols",
+        "dev.capabilities",
+        "dev.capture",
         "dev.control",
         "dev.copy",
         "dev.copy2eps",
         "dev.copy2pdf",
         "dev.cur",
+        "dev.flush",
+        "dev.hold",
         "dev.interactive",
         "dev.list",
         "dev.new",
@@ -466,6 +473,7 @@
         "plot",
         "plot.default",
         "plot.design",
+        "plot.function",
         "plot.new",
         "plot.window",
         "plot.xy",
@@ -675,6 +683,7 @@
         "friedman.test",
         "ftable",
         "gaussian",
+        "getCall",
         "getInitial",
         "get_all_vars",
         "glm",
@@ -1030,7 +1039,9 @@
         "canCoerce",
         "cbind2",
         "checkSlotAssignment",
+        "classLabel",
         "classMetaName",
+        "className",
         "classesToAM",
         "coerce",
         "coerce<-",
@@ -1132,6 +1143,7 @@
         "methodsPackageMetaName",
         "missingArg",
         "mlistMetaName",
+        "multipleClasses",
         "new",
         "newBasic",
         "newClassRepresentation",
