@@ -5,6 +5,9 @@
 ExcludeVars <- c("R_SESSION_TMPDIR","R_HISTFILE")
 IncludeVars <- Sys.getenv()
 IncludeVars <- IncludeVars[grep("^R_",names(IncludeVars),perl=TRUE)]
+if (.Platform$OS.type == "windows") {
+    IncludeVars <- gsub("\\\\", "/", IncludeVars, perl=TRUE)
+}
 cat("    const char *R_VARS[] = {\n")
 for (i in 1:length(IncludeVars)){
 	if (names(IncludeVars)[i] %in% ExcludeVars)
